using Foundation;
using MvvmCross.iOS.Views;
using System;
using UIKit;
using Tackzee.Rider.Core.ViewModels;
using MvvmCross.Binding.BindingContext;
using CoreLocation;
using Tackzee.PlatformShare.Core.Models;
using MapboxNavigation;
using Mapbox;
using MapboxDirections;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace Tackzee_Rider.iOS
{
	[MvxFromStoryboard("RideStb")]
	public partial class HomeController : MvxViewController<HomeViewModel>, IMGLMapViewDelegate
	{
		public AddOnsModel AddOns;
		CLLocationCoordinate2D centerNantucket = new CLLocationCoordinate2D(41.276750, -70.090889);
		CLLocationCoordinate2D currentPosition = new CLLocationCoordinate2D();
		LocationModel origin, destination;
		MGLPointAnnotation originMarker, destinationMarker;

		public HomeController(IntPtr handle) : base(handle)
		{
		}

		public HomeController() : base("HomeController", null)
		{
		}

		public override void ViewDidLoad()
		{
			base.ViewDidLoad();

			InitializeControls();
		}

		public override async void ViewWillAppear(bool animated)
		{
			if (!ViewModel.Trip.OriginDestination.HasOriginCoordinates || !ViewModel.Trip.OriginDestination.HasDestinationCoordinates)
			{
				ResumedOptionsConstraint.Constant = -50;
				ExpandedOptionsConstraint.Constant = -100;
				ExpandedOptionsHeight.Constant = 0;
				return;
			}

			ResumedOptionsConstraint.Constant = 0;
			ExpandedOptionsConstraint.Constant = -100;
			ExpandedOptionsHeight.Constant = 0;
			UIView.Animate(0.3, View.LayoutIfNeeded);
			CalculateRoute();
			await CalculateFare();
		}

		async Task CalculateFare()
		{
			await ViewModel.CalculateFare();
		}

		void CalculateRoute()
		{
			if (ViewModel.Trip.OriginDestination.HasOriginCoordinates && ViewModel.Trip.OriginDestination.HasDestinationCoordinates)
			{
				origin = ViewModel.Trip.OriginDestination.Origin;
				destination = ViewModel.Trip.OriginDestination.Destination;
				var originCoordinate = new CLLocationCoordinate2D(origin.Latitude, origin.Longitude);
				originMarker = new MGLPointAnnotation
				{
					Coordinate = originCoordinate,
					Title = origin.FullAddress
				};
				var destinationCoordinate = new CLLocationCoordinate2D(destination.Latitude, destination.Longitude);
				destinationMarker = new MGLPointAnnotation
				{
					Coordinate = destinationCoordinate,
					Title = destination.FullAddress
				};

				mapView.AddAnnotation(originMarker);
				mapView.AddAnnotation(destinationMarker);

				//var originWaypoint = new MBWaypoint(originMarker.Coordinate, -1, originMarker.Title);
				//var destinationWaypoint = new MBWaypoint(destinationMarker.Coordinate, -1, destinationMarker.Title);

				//var locations = new List<MBWaypoint>
				//{
				//	originWaypoint,
				//	destinationWaypoint
				//};
				//var options = new MBRouteOptions(locations.ToArray(), "automobileAvoidingTraffic");

				//MBDirections.SharedDirections.CalculateDirectionsWithOptions(options, HandleAction);
			}
		}

		//void HandleAction(MBWaypoint[] waypoints, MBRoute[] routes, NSError error)
		//{
		//	var finalRoute = routes[0];
		//	var polyline = new MGLPolylineFeature();
		//	CLLocationCoordinate2D coordinates;
		//	finalRoute.GetCoordinates(out coordinates);
		//	polyline.SetCoordinates(ref coordinates, finalRoute.CoordinateCount);

		//	if (mapView.Style.Sources != null)
		//	{
		//		var style = mapView.Style.SourceWithIdentifier("route-source") as MGLShapeSource;
		//		style.Shape = polyline;
		//	}
		//	else
		//	{
		//		var source = new MGLShapeSource("route-source", polyline, null);
		//		var lineStyle = new MGLLineStyleLayer("route-source", source);
		//		lineStyle.LineColor = MGLStyleValue.ValueWithRawValue(new UIColor((System.nfloat)0.1897518039, (System.nfloat)0.3010634184, (System.nfloat)0.7994888425, 1));

		//		mapView.Style.AddSource(source);
		//		mapView.Style.AddLayer(lineStyle);
		//	}

		//}

		void UpdateMoreOptions()
		{
			throw new NotImplementedException();
		}

		void ViewModel_OnLocationChanged(double latitude, double longitude)
		{
			UpdateCameraPosition(new CLLocationCoordinate2D(latitude, longitude));
		}

		void ViewModel_OnChangedOptionValue(bool isExpanded)
		{
			ResumedOptionsConstraint.Constant = isExpanded ? 0 : -50;
			ExpandedOptionsConstraint.Constant = isExpanded ? -100 : 0;
			ExpandedOptionsHeight.Constant = isExpanded ? 0 : 149;
			UIView.Animate(0.3, () => { 
				View.LayoutIfNeeded();
			});
		}

		void InitializeControls()
		{
			var set = this.CreateBindingSet<HomeController, HomeViewModel>();

			set.Bind(Button).To(viewModel => viewModel.SetLocationCommand);
			set.Bind(MoreOptionsButton).To(viewModel => viewModel.ShowMoreOptionsListCommand);
			set.Bind(CollapseButton).To(viewModel => viewModel.HideMoreOptionsListCommand);
			set.Bind(OriginTextField).To(viewModel => viewModel.Trip.OriginDestination.Origin.FullAddress);
			set.Bind(DestinationTextField).To(viewModel => viewModel.Trip.OriginDestination.Destination.FullAddress);
			set.Bind(TravelFare).To(viewModel => viewModel.Trip.BaseFare).WithConversion("StringFormat", "$ 0.00");
			set.Bind(PaymentAmount).To(viewModel => viewModel.Trip.BaseFare).WithConversion("StringFormat", "$ 0.00");
			set.Apply();

			ViewModel.OnChangedOptionValue += ViewModel_OnChangedOptionValue;
			ViewModel.OnLocationChanged += ViewModel_OnLocationChanged;
			ViewModel.EnableMoreOptions = false;
			ViewModel.EnableMoreOptionsList = false;
			ViewModel.UpdateMoreOptions += UpdateMoreOptions;
			AddOns = new AddOnsModel(0, 0, 0, 0, string.Empty);

			OriginTextField.Font = UIFont.FromName("Montserrat-Regular", 16f);
			OriginTextField.EditingDidBegin += TextField_EditingDidBegin;
			DestinationTextField.Font = UIFont.FromName("Montserrat-Regular", 16f);
			DestinationTextField.EditingDidBegin += TextField_EditingDidBegin;

			mapView.AutoresizingMask = UIViewAutoresizing.FlexibleDimensions;

			if (currentPosition.Latitude != 0 && currentPosition.Longitude != 0)
			{
				var latLng = new CLLocationCoordinate2D(currentPosition.Latitude, currentPosition.Longitude);
				UpdateCameraPosition(latLng);
			}
			else
			{
				UpdateCameraPosition(centerNantucket);
			}

			mapView.ShowsUserLocation = true;
			mapView.SetUserTrackingMode(MGLUserTrackingMode.FollowWithHeading, true);
		}

		void TextField_EditingDidBegin(object sender, EventArgs e)
		{
			ViewModel.SetupOriginDestination();
			((UITextField)sender).ResignFirstResponder();
		}

		void UpdateCameraPosition(CLLocationCoordinate2D latLng)
		{
			mapView.SetCenterCoordinate(latLng, 13, true);
		}


	}
}